﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ALPS.SteadyState {
  [Item("SteadyStateMatingPoolCreator", "")]
  [StorableClass]
  public class SteadyStateMatingPoolCreator : SingleSuccessorOperator {

    [StorableConstructor]
    private SteadyStateMatingPoolCreator(bool deserializing)
      : base(deserializing) { }
    private SteadyStateMatingPoolCreator(SteadyStateMatingPoolCreator original, Cloner cloner)
      : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new SteadyStateMatingPoolCreator(this, cloner);
    }
    public SteadyStateMatingPoolCreator()
      : base() {
    }

    public override IOperation Apply() {
      var matingPool = ExecutionContext.Scope;
      var currentLayer = matingPool.Parent;
      int layerNumber = ((IntValue)currentLayer.Variables["Layer"].Value).Value;

      if (layerNumber > 0) {
        var prevLayer = currentLayer.Parent.SubScopes[layerNumber - 1];
        var prevLayerIndividuals = prevLayer.SubScopes;

        foreach (var individual in prevLayerIndividuals) {
          matingPool.SubScopes.Add((IScope)individual.Clone());
        }
      }

      return base.Apply();
    }
  }
}